/*
 * Parse command line arguments.
 */

#include <stdio.h>
#include <string.h>

#define BUFLEN 1024

%% params
	# A buffer to collect argurments
	struct {
		char buffer1[BUFLEN+1];
		char buffer2[BUFLEN+1];
		int buflen1;
		int buflen2;
	};

	# Init the buffer.
	init {
		fsm->buflen1 = 0;
		fsm->buflen2 = 0;
	}

	# Append to the buffer.
	action b1_append {
		if ( fsm->buflen1 < BUFLEN )
			fsm->buffer1[fsm->buflen1++] = fc;
	}
	action b2_append {
		if ( fsm->buflen2 < BUFLEN )
			fsm->buffer2[fsm->buflen2++] = fc;
	}

	# Terminate a buffer.
	action b1_term {
		if ( fsm->buflen1 < BUFLEN )
			fsm->buffer1[fsm->buflen1++] = 0;
	}
	action b2_term {
		if ( fsm->buflen2 < BUFLEN )
			fsm->buffer2[fsm->buflen2++] = 0;
	}

	# Clear out the buffer
	action b1_clear { fsm->buflen1 = 0; }
	action b2_clear { fsm->buflen2 = 0; }

	action help { printf("help\n"); }
	action version { printf("version\n"); }
	action output { printf("output: \"%s\"\n", fsm->buffer1); }
	action spec { printf("spec: \"%s\"\n", fsm->buffer1); }
	action machine { printf("machine: \"%s\" :: \"%s\"\n", fsm->buffer1, fsm->buffer2); }

	# Helpers that collect strings
	string = [^\0]+ >b1_clear $b1_append %b1_term;
	mstring1 = [^\0:]+ >b1_clear $b1_append %b1_term;
	mstring2 = [^\0:]+ >b2_clear $b2_append %b2_term;

	# Different arguments.
	help = ( '-h' | '-H' | '-?' | '--help' ) 0 @help;
	version = ( '-v' | '--version' ) 0 @version;
	output = ( '-o' 0? string ) 0 @output;
	spec = ( '-M' 0? mstring1 ) 0 @spec;
	machine = ( '-M' 0? mstring1 '::' mstring2 ) 0 @machine;

	main := ( 
		help | 
		version | 
		output |
		spec |
		machine
	)*;
%%

#define BUFSIZE 2048

int main( int argc, char **argv )
{
	int a;
	struct params params;

	params_init( &params );
	for ( a = 1; a < argc; a++ )
		params_execute( &params, argv[a], strlen(argv[a])+1 );
	if ( params_finish( &params ) != 1 )
		fprintf( stderr, "params: error processing arguments\n" );

	return 0;
}
